-- pixel.lua : a pixel class capable of reprsenting
-- and manipulating colors in PC-space (gamma=2.2) or
-- in linear space (gamma=1).
--
-- Version 1.0 
-- 
-- Copyright 2016 by Samuel Devulder
--
-- This program is free software; you can redistribute
-- it and/or modify it under the terms of the GNU
-- General Public License as published by the Free
-- Software Foundation; version 2 of the License.
-- See <http://www.gnu.org/licenses/>


if not Pixel then
	Pixel = {}
	function Pixel:new(r,g,b)
		local o = {};
		o.r = type(r)=='number' and r or r and r.r or 0;
		o.g = type(g)=='number' and g or r and r.g or 0;
		o.b = type(b)=='number' and b or r and r.b or 0;
		setmetatable(o, self)
		self.__index = self
		return o
	end
	Pixel.black = Pixel:new(0,0,0)

	function Pixel:clone()
		return Pixel:new(self.r, self.g, self.b)
	end

	function Pixel:__tostring()
		return "(r=" .. self.r .. " g=" .. self.g .. " b=" .. self.b .. ")"
	end

	function Pixel:mul(val)
		self.r = self.r * val;
		self.g = self.g * val;
		self.b = self.b * val;
		return self;
	end

	function Pixel:div(val)
		return self:mul(1/val);
	end

	function Pixel:add(other)
		self.r = self.r + other.r;
		self.g = self.g + other.g;
		self.b = self.b + other.b;
		return self;
	end
	
	function Pixel:sub(other)
		self.r = self.r - other.r;
		self.g = self.g - other.g;
		self.b = self.b - other.b;
		return self;
	end
	
	function Pixel:floor() 
		self.r = math.min(math.floor(self.r),255);
		self.g = math.min(math.floor(self.g),255);
		self.b = math.min(math.floor(self.b),255);
		return self;
	end

	function Pixel:toPC() 
		local function f(val) 
			val = val/255
			if val<=0.018 then val = 4.5*val; else val = 1.099*(val ^ (1/2.2))-0.099; end
			return val*255
		end;
		self.r = f(self.r);
		self.g = f(self.g);
		self.b = f(self.b);
		return self;
	end

	function Pixel:toLinear() 
		local function f(val) 
			val = val/255
			if val<=0.081 then val = val/4.5; else val = ((val+0.099)/1.099)^2.2; end
			return val*255
		end;
		self.r = f(self.r);
		self.g = f(self.g);
		self.b = f(self.b);
		return self;
	end

	function Pixel:toTuple()
		return self.r, self.g, self.b
	end


	-- return the Pixel @(x,y) on the original screen in linear space
	local screen_w, screen_h, _getLinearPicturePixel = getpicturesize()
	function getLinearPicturePixel(x,y) 
		if _getLinearPicturePixel==nil then
			_getLinearPicturePixel = {}
			for i=0,255 do _getLinearPicturePixel[i] = Pixel:new(getbackupcolor(i)):toLinear(); end
			end
		return (x<0 or y<0 or x>=screen_w or y>=screen_h) and Pixel.black or _getLinearPicturePixel[getbackuppixel(x,y)]
	end
	
end -- Pixel defined
